#--reworked 7/2018 for scenario emissions, ThL
#--updated on 5/2/2018 with new paths, ThL
#--updated on 4/5/2017 with improved Sheng & Zwiers algorithm, ThL
#--corrected some interpolation preprocessing (compared to v4)
#--updated on 5/5/2017 with correction factor (30/46) on NOx vs. NO
#--updated on 9/5/2017 with output separation BB/anthro for SO2, NOx and NH3
#--corrected on 22/6/2017 for BB: undef values zeroed before remapping
#--corrected on 26/09/2017 for NOx units: PNNL dataset is kg NO2, VUA is kg NO. 
#  INCA expects NO in its AER version.
#
#INCA  conc_dms flx_nox flx_bc flx_pom flx_bbbc flx_bbpom flx_so2 flx_so4 flx_h2s flx_nh3
#CMIP6 species  NOx     BC     OC                         SO2                     NH3
# + NMVOC CO

#--INCA example file where dms_conc can be reused
fileINCAex="/home/oboucher/CMIP6/AER_EMISSIONS/INCAfile/sflx_lmdz_phy_1997.nc"

#--input directory for emissions
dirin="/prodigfs/project/input4MIPs/CMIP6/ScenarioMIP/IAMC/"

#--LMDz grid information
grid="144x143"
gridfile="../GRID/grid-lmdz-lonlat_"${grid}
nbpoint=$((144*141+2))

for scen in "ssp119" "ssp126" "ssp245" "ssp370" "ssp434" "ssp460" "ssp534-over" "ssp585"
do

#--output directory
dirout="/data/"${USER}"/CMIP6/AEROSOL/ScenarioMIP/${scen}/v1/"
if [ ! -d ${dirout} ] ; then mkdir -p ${dirout} ; fi

if [ ${scen} == "ssp119" ] ; then prefix="IMAGE" ; fi
if [ ${scen} == "ssp126" ] ; then prefix="IMAGE" ; fi
if [ ${scen} == "ssp245" ] ; then prefix="MESSAGE-GLOBIOM" ; fi
if [ ${scen} == "ssp370" ] ; then prefix="AIM" ; fi
if [ ${scen} == "ssp434" ] ; then prefix="GCAM4" ; fi
if [ ${scen} == "ssp460" ] ; then prefix="GCAM4" ; fi
if [ ${scen} == "ssp534-over" ] ; then prefix="REMIND-MAGPIE" ; fi
if [ ${scen} == "ssp585" ] ; then prefix="REMIND-MAGPIE" ; fi

#--Unique file for the whole period, so fixed variables:
year1=2015
year2=2100

#--Loop on species (anthro)
for species in "BC" "NOx" "OC" "SO2" "NH3"
do

echo "... "${year}" : Dealing with "${species}" anthro input file..."

if [ ${species} = "BC"  ] ; then speciesinca="bc"  ; fi
if [ ${species} = "NOx" ] ; then speciesinca="nox" ; fi
if [ ${species} = "OC"  ] ; then speciesinca="pom" ; fi
if [ ${species} = "SO2" ] ; then speciesinca="so2" ; fi
if [ ${species} = "NH3" ] ; then speciesinca="nh3" ; fi

if [ ${species} = "BC"  ] ; then speciesUp="BC"  ; fi
if [ ${species} = "NOx" ] ; then speciesUp="NOX" ; fi
if [ ${species} = "OC"  ] ; then speciesUp="OC" ; fi
if [ ${species} = "SO2" ] ; then speciesUp="SO2" ; fi
if [ ${species} = "NH3" ] ; then speciesUp="NH3" ; fi

#--input file anthro
filename=${dirin}IAMC-${prefix}-${scen}-1-1/atmos/mon/${species}_em_anthro/gn/v20180628/${species}-em-anthro_input4MIPs_emissions_ScenarioMIP_IAMC-${prefix}-${scen}-1-1_gn_${year1}01-${year2}12.nc

#--input file summed over sectors
filesum=${dirout}${species}-em-anthro_input4MIPs_emissions_ScenarioMIP_IAMC-${prefix}-${scen}-1-1_gn_${year1}01-${year2}12_sum.nc
#--reworked input file
filetmp=${dirout}${species}-em-anthro_input4MIPs_emissions_ScenarioMIP_IAMC-${prefix}-${scen}-1-1_gn_${year1}01-${year2}12_tmp.nc
#--time-interpolated input file
fileintanthro=${dirout}${species}-em-anthro_input4MIPs_emissions_ScenarioMIP_IAMC-${prefix}-${scen}-1-1_gn_${year1}01-${year2}12_interp.nc

if [ -f ${fileintanthro} ]
then
   echo "...Nothing to do!"
else

rm -f ${filesum} ${filetmp} ${fileintanthro}

#--re-formatting the input file
#--summing over sectors
rm -f sumsectors.jnl
cat << eod > sumsectors.jnl
set memory/size=100
use "${filename}"
save/clobber/file="${filesum}" ${species}_EM_ANTHRO[k=@sum]
eod
#--run ferret script
ferret << eod 
go sumsectors.jnl
exit
eod

#--reformatting input file for IDL to be able to read
ncks -3 ${filesum} ${filetmp}
#--cleaning up
rm -f ${filesum}

#--IDL code for time interpolation, to fill in 9-year gaps in input file
rm -f time_interp_over_gaps.pro
cat << eod >> time_interp_over_gaps.pro
pro time_interp_over_gaps

; Les dimensions des fichiers
nlon=720
nlat=360
nt=1032

une_annee=[31,30,29,31,30,31,30,31,31,30,31,30]
les_mois = [31,28,31,30,31,30,31,31,30,31,30,31]

jours_in = [15, 45, 74, 105, 135, 166, 196, 227, 258, 288, 319, 349, 1840, 1870, $
    1899, 1930, 1960, 1991, 2021, 2052, 2083, 2113, 2144, 2174, 5490, 5520, $
    5549, 5580, 5610, 5641, 5671, 5702, 5733, 5763, 5794, 5824, 9140, 9170, $
    9199, 9230, 9260, 9291, 9321, 9352, 9383, 9413, 9444, 9474, 12790, 12820, $
    12849, 12880, 12910, 12941, 12971, 13002, 13033, 13063, 13094, 13124, $
    16440, 16470, 16499, 16530, 16560, 16591, 16621, 16652, 16683, 16713, $
    16744, 16774, 20090, 20120, 20149, 20180, 20210, 20241, 20271, 20302, $
    20333, 20363, 20394, 20424, 23740, 23770, 23799, 23830, 23860, 23891, $
    23921, 23952, 23983, 24013, 24044, 24074, 27390, 27420, 27449, 27480, $ 
    27510, 27541, 27571, 27602, 27633, 27663, 27694, 27724, 31040, 31070, $ 
    31099, 31130, 31160, 31191, 31221, 31252, 31283, 31313, 31344, 31374]

jours_out=make_array(1032)
jours_out(0) = 15
for cpt=1,1031 do jours_out(cpt) = jours_out(cpt-1) + une_annee(cpt mod 12)

idin = ncdf_open('${filetmp}')
varid = ncdf_varid(idin,'${speciesUp}_EM_ANTHRO')
lonid = ncdf_varid(idin,'LON')
latid = ncdf_varid(idin,'LAT')
;lonbndid = ncdf_varid(idin,'LON_bnds')
;latbndid = ncdf_varid(idin,'LAT_bnds')
timbndid = ncdf_varid(idin,'TIME_bnds')
ncdf_varget, idin, varid, emiss_init
ncdf_attget, idin, varid, "units", emiss_unit
ncdf_attget, idin, varid, "long_name", emiss_longname
ncdf_varget, idin, lonid, lon
ncdf_varget, idin, latid, lat
;ncdf_varget, idin, lonbndid, lonbnds
;ncdf_varget, idin, latbndid, latbnds
ncdf_varget, idin, timbndid, timbnds_in
ncdf_close, idin

timbnds_out = make_array(2,1032)
timbnds_out(*,0) = [0,31]
timbnds_out(*,1) = [31,59]
for cpt=2,1031 do timbnds_out(0,cpt) = timbnds_out(0,cpt-1) + les_mois((cpt-1) mod 12)
for cpt=1,1030 do timbnds_out(1,cpt) = timbnds_out(0,cpt+1)
timbnds_out(1,1031) = 31390

emiss_interpol=make_array(nlon,nlat,1032,value=0.)

indices_out_mois = make_array(86,12,value=0)
for m=0,11 do begin
   for n=0,85 do indices_out_mois(n,m) = 12*n + m
endfor

jours_out_mois = make_array(86,12,value=0)
for n=0,85 do begin
   for m=0,11 do jours_out_mois(n,m) = jours_out(indices_out_mois(n,m))
endfor

jours_in_mois = make_array(n_elements(jours_in)/12,12,value=0)
for m=0,11 do begin
   for n=0,n_elements(jours_in)/12-1 do jours_in_mois(n,m) = jours_in(n*12 + m)
endfor

indices_in_mois = make_array(n_elements(jours_in)/12,12,value=0)
for m=0,11 do begin
   for n=0,n_elements(jours_in)/12-1 do indices_in_mois(n,m) = where(jours_in eq jours_in_mois(n,m))
endfor

; Interpolation mois par mois...
for i=0,nlon-1 do begin
   for j=0,nlat-1 do begin
      for m=0,11 do begin
         emiss_interpol(i,j,indices_out_mois(*,m)) = interpol(emiss_init(i,j,indices_in_mois(*,m)),jours_in_mois(*,m),jours_out_mois(*,m))
      endfor
   endfor
endfor

idout = ncdf_create('${fileintanthro}', /clobber)
dimlonid = ncdf_dimdef(idout,'lon',nlon)
dimlatid = ncdf_dimdef(idout,'lat',nlat)
dimtimid = ncdf_dimdef(idout,'time',1032)
dimbndid = ncdf_dimdef(idout,'bound',2)
varlonid = ncdf_vardef(idout, 'lon', [dimlonid], /double)
varlatid = ncdf_vardef(idout, 'lat', [dimlatid], /double)
vartimid = ncdf_vardef(idout, 'time', [dimtimid], /double)
;varlonbndsid = ncdf_vardef(idout, 'lon_bnds', [dimbndid,dimlonid], /double)
;varlatbndsid = ncdf_vardef(idout, 'lat_bnds', [dimbndid,dimlatid], /double)
vartimbndsid = ncdf_vardef(idout, 'time_bnds', [dimbndid,dimtimid], /double)
varemissid = ncdf_vardef(idout, '${species}_em_anthro', [dimlonid,dimlatid,dimtimid], /float)
ncdf_control, idout, /endef
ncdf_varput, idout, varlonid, lon
ncdf_varput, idout, varlatid, lat
;ncdf_varput, idout, varlonbndsid, lonbnds
;ncdf_varput, idout, varlatbndsid, latbnds
ncdf_varput, idout, vartimbndsid, timbnds_out
ncdf_varput, idout, varemissid, emiss_interpol
ncdf_varput, idout, vartimid, jours_out
ncdf_control, idout, /redef
ncdf_attput, idout, varlonid, "units", "degrees_east"
ncdf_attput, idout, varlonid, "long_name", "longitude"
ncdf_attput, idout, varlonid, "axis", "X"
ncdf_attput, idout, varlonid, "bounds", "lon_bnds"
ncdf_attput, idout, varlonid, "modulo", 360.
ncdf_attput, idout, varlonid, "realtopology", "circular"
ncdf_attput, idout, varlonid, "standard_name", "longitude"
ncdf_attput, idout, varlonid, "topology", "circular"
ncdf_attput, idout, varlatid, "units", "degrees_north"
ncdf_attput, idout, varlatid, "long_name", "latitude"
ncdf_attput, idout, varlatid, "axis", "Y"
ncdf_attput, idout, varlatid, "bounds", "lat_bnds"
ncdf_attput, idout, varlatid, "realtopology", "linear"
ncdf_attput, idout, varlatid, "standard_name", "latitude"
ncdf_attput, idout, varemissid, "units", string(emiss_unit)
ncdf_attput, idout, varemissid, "_FillValue", 1.e+20
ncdf_attput, idout, varemissid, "cell_methods", "time: mean"
ncdf_attput, idout, varemissid, "long_name", string(emiss_longname)
ncdf_attput, idout, varemissid, "missing_value", 1.e+20
ncdf_attput, idout, vartimid, "units", "days since 2015-01-01 0:0:0"
ncdf_attput, idout, vartimid, "long_name", "time"
ncdf_attput, idout, vartimid, "calendar", '365_day'
ncdf_attput, idout, vartimid, "axis", "T"
ncdf_attput, idout, vartimid, "bounds", "time_bnds"
ncdf_attput, idout, vartimid, "realtopology", "linear"
ncdf_attput, idout, vartimid, "standard_name", "time"
ncdf_close, idout
end
eod
#
#--calling IDL
#
/opt/idl-6.4/idl/bin/idl << eod
.r time_interp_over_gaps.pro
time_interp_over_gaps
exit
eod
#

#--adding lat_bnds and lon_bnds variables
ncks -A -v lat_bnds,lon_bnds ${filename} ${fileintanthro}

#--cleaning up
rm -f ${filetmp}

#--end test whether file tmp already exists
fi



#--now dealing with the BB input file

echo "... "${year}" : Dealing with "${species}" openburning input file..."

#--input file BB
filename=${dirin}IAMC-${prefix}-${scen}-1-1/atmos/mon/${species}_em_openburning/gn/v20180628/${species}-em-openburning_input4MIPs_emissions_ScenarioMIP_IAMC-${prefix}-${scen}-1-1_gn_${year1}01-${year2}12.nc
#--reworked input file
filetmp=${dirout}${species}-em-openburning_input4MIPs_emissions_ScenarioMIP_IAMC-${prefix}-${scen}-1-1_gn_${year1}01-${year2}12_tmp.nc
#--time-interpolated input file
fileintbb=${dirout}${species}-em-openburning_input4MIPs_emissions_ScenarioMIP_IAMC-${prefix}-${scen}-1-1_gn_${year1}01-${year2}12_interp.nc

if [ -f ${fileintbb} ]
then
   echo "...Nothing to do!"
else

rm -f ${filetmp} ${fileintbb}

#--no need to sum over sectors, as none in BB files

#--reformatting input file for IDL to be able to read
ncks -3 ${filename} ${filetmp}

#--IDL code for time interpolation, to fill in 9-year gaps in input file
rm -f time_interp_over_gaps.pro
cat << eod >> time_interp_over_gaps.pro
pro time_interp_over_gaps

; Les dimensions des fichiers
nlon=720
nlat=360
nt=1032

une_annee=[31,30,29,31,30,31,30,31,31,30,31,30]
les_mois = [31,28,31,30,31,30,31,31,30,31,30,31]

jours_in = [15, 45, 74, 105, 135, 166, 196, 227, 258, 288, 319, 349, 1840, 1870, $
    1899, 1930, 1960, 1991, 2021, 2052, 2083, 2113, 2144, 2174, 5490, 5520, $
    5549, 5580, 5610, 5641, 5671, 5702, 5733, 5763, 5794, 5824, 9140, 9170, $
    9199, 9230, 9260, 9291, 9321, 9352, 9383, 9413, 9444, 9474, 12790, 12820, $
    12849, 12880, 12910, 12941, 12971, 13002, 13033, 13063, 13094, 13124, $
    16440, 16470, 16499, 16530, 16560, 16591, 16621, 16652, 16683, 16713, $
    16744, 16774, 20090, 20120, 20149, 20180, 20210, 20241, 20271, 20302, $
    20333, 20363, 20394, 20424, 23740, 23770, 23799, 23830, 23860, 23891, $
    23921, 23952, 23983, 24013, 24044, 24074, 27390, 27420, 27449, 27480, $ 
    27510, 27541, 27571, 27602, 27633, 27663, 27694, 27724, 31040, 31070, $ 
    31099, 31130, 31160, 31191, 31221, 31252, 31283, 31313, 31344, 31374]

jours_out=make_array(1032)
jours_out(0) = 15
for cpt=1,1031 do jours_out(cpt) = jours_out(cpt-1) + une_annee(cpt mod 12)

idin = ncdf_open('${filetmp}')
varid = ncdf_varid(idin,'${species}_em_openburning')      ; Here, small caps
lonid = ncdf_varid(idin,'lon')
latid = ncdf_varid(idin,'lat')
;lonbndid = ncdf_varid(idin,'lon_bnds')
;latbndid = ncdf_varid(idin,'lat_bnds')
timbndid = ncdf_varid(idin,'time_bnds')
ncdf_varget, idin, varid, emiss_init
ncdf_attget, idin, varid, "units", emiss_unit
ncdf_attget, idin, varid, "long_name", emiss_longname
ncdf_varget, idin, lonid, lon
ncdf_varget, idin, latid, lat
;ncdf_varget, idin, lonbndid, lonbnds
;ncdf_varget, idin, latbndid, latbnds
ncdf_varget, idin, timbndid, timbnds_in
ncdf_close, idin

timbnds_out = make_array(2,1032)
timbnds_out(*,0) = [0,31]
timbnds_out(*,1) = [31,59]
for cpt=2,1031 do timbnds_out(0,cpt) = timbnds_out(0,cpt-1) + une_annee((cpt-1) mod 12)
for cpt=1,1030 do timbnds_out(1,cpt) = timbnds_out(0,cpt+1)
timbnds_out(1,1031) = 31390

emiss_interpol=make_array(nlon,nlat,1032,value=0.)

indices_out_mois = make_array(86,12,value=0)
for m=0,11 do begin
   for n=0,85 do indices_out_mois(n,m) = 12*n + m
endfor

jours_out_mois = make_array(86,12,value=0)
for n=0,85 do begin
   for m=0,11 do jours_out_mois(n,m) = jours_out(indices_out_mois(n,m))
endfor

jours_in_mois = make_array(n_elements(jours_in)/12,12,value=0)
for m=0,11 do begin
   for n=0,n_elements(jours_in)/12-1 do jours_in_mois(n,m) = jours_in(n*12 + m)
endfor

indices_in_mois = make_array(n_elements(jours_in)/12,12,value=0)
for m=0,11 do begin
   for n=0,n_elements(jours_in)/12-1 do indices_in_mois(n,m) = where(jours_in eq jours_in_mois(n,m))
endfor

; Interpolation mois par mois...
for i=0,nlon-1 do begin
   for j=0,nlat-1 do begin
      for m=0,11 do begin
         emiss_interpol(i,j,indices_out_mois(*,m)) = interpol(emiss_init(i,j,indices_in_mois(*,m)),jours_in_mois(*,m),jours_out_mois(*,m))
      endfor
   endfor
endfor

idout = ncdf_create('${fileintbb}', /clobber)
dimlonid = ncdf_dimdef(idout,'lon',nlon)
dimlatid = ncdf_dimdef(idout,'lat',nlat)
dimtimid = ncdf_dimdef(idout,'time',1032)
dimbndid = ncdf_dimdef(idout,'bound',2)
varlonid = ncdf_vardef(idout, 'lon', [dimlonid], /double)
varlatid = ncdf_vardef(idout, 'lat', [dimlatid], /double)
vartimid = ncdf_vardef(idout, 'time', [dimtimid], /double)
;varlonbndsid = ncdf_vardef(idout, 'lon_bnds', [dimbndid,dimlonid], /double)
;varlatbndsid = ncdf_vardef(idout, 'lat_bnds', [dimbndid,dimlatid], /double)
vartimbndsid = ncdf_vardef(idout, 'time_bnds', [dimbndid,dimtimid], /double)
varemissid = ncdf_vardef(idout, '${species}_em_openburning', [dimlonid,dimlatid,dimtimid], /float)
ncdf_control, idout, /endef
ncdf_varput, idout, varlonid, lon
ncdf_varput, idout, varlatid, lat
;ncdf_varput, idout, varlonbndsid, lonbnds
;ncdf_varput, idout, varlatbndsid, latbnds
ncdf_varput, idout, vartimbndsid, timbnds_out
ncdf_varput, idout, varemissid, emiss_interpol
ncdf_varput, idout, vartimid, jours_out
ncdf_control, idout, /redef
ncdf_attput, idout, varlonid, "units", "degrees_east"
ncdf_attput, idout, varlonid, "long_name", "longitude"
ncdf_attput, idout, varlonid, "axis", "X"
ncdf_attput, idout, varlonid, "bounds", "lon_bnds"
ncdf_attput, idout, varlonid, "modulo", 360.
ncdf_attput, idout, varlonid, "realtopology", "circular"
ncdf_attput, idout, varlonid, "standard_name", "longitude"
ncdf_attput, idout, varlonid, "topology", "circular"
ncdf_attput, idout, varlatid, "units", "degrees_north"
ncdf_attput, idout, varlatid, "long_name", "latitude"
ncdf_attput, idout, varlatid, "axis", "Y"
ncdf_attput, idout, varlatid, "bounds", "lat_bnds"
ncdf_attput, idout, varlatid, "realtopology", "linear"
ncdf_attput, idout, varlatid, "standard_name", "latitude"
ncdf_attput, idout, varemissid, "units", string(emiss_unit)
ncdf_attput, idout, varemissid, "_FillValue", 1.e+20
ncdf_attput, idout, varemissid, "cell_methods", "time: mean"
ncdf_attput, idout, varemissid, "long_name", string(emiss_longname)
ncdf_attput, idout, varemissid, "missing_value", 1.e+20
ncdf_attput, idout, vartimid, "units", "days since 2015-01-01 0:0:0"
ncdf_attput, idout, vartimid, "long_name", "time"
ncdf_attput, idout, vartimid, "calendar", '365_day'
ncdf_attput, idout, vartimid, "axis", "T"
ncdf_attput, idout, vartimid, "bounds", "time_bnds"
ncdf_attput, idout, vartimid, "realtopology", "linear"
ncdf_attput, idout, vartimid, "standard_name", "time"
ncdf_close, idout
end
eod
#
#--calling IDL
#
/opt/idl-6.4/idl/bin/idl << eod
.r time_interp_over_gaps.pro
time_interp_over_gaps
exit
eod
#

#--adding lat_bnds and lon_bnds variables
ncks -A -v lat_bnds,lon_bnds ${filename} ${fileintbb}

#--cleaning up
rm -f ${filetmp}

#--end test if interpolated file already exists
fi

#--end loop on species
done



#--anthro and BB input files fileintanthro and fileintbb are now ready, for the whole period and for the current scenario
#--new double loop on species and years

#--loop on years
for year in {2015..2100}
do

#--loop on species
for species in "BC" "NOx" "OC" "SO2" "NH3"
do

if [ ${species} = "BC"  ] ; then speciesinca="bc"  ; fi
if [ ${species} = "NOx" ] ; then speciesinca="nox" ; fi
if [ ${species} = "OC"  ] ; then speciesinca="pom" ; fi
if [ ${species} = "SO2" ] ; then speciesinca="so2" ; fi
if [ ${species} = "NH3" ] ; then speciesinca="nh3" ; fi

#--dealing with anthro

#--input file
fileintanthro=${dirout}${species}-em-anthro_input4MIPs_emissions_ScenarioMIP_IAMC-${prefix}-${scen}-1-1_gn_${year1}01-${year2}12_interp.nc
#--output files
filenameout1=${dirout}flux_${speciesinca}_${year}.nc
filenameout1b=${dirout}flux_0_${speciesinca}_${year}.nc
filenameout2=${dirout}flux_lmdz_${speciesinca}_${year}.nc
filenameout3=${dirout}flux_vector_${speciesinca}_${year}.nc

echo ${fileintanthro} ${filenameout1} ${filenameout2} ${filenameout3}
rm -f ${filenameout1} ${filenameout2} ${filenameout3}

#--extracting the correct year
rm -f select_yr.jnl
cat << eod > select_yr.jnl
set memory/size=100
use "${fileintanthro}"
set region/t=16-jan-${year}:16-dec-${year}
save/clobber/file="${filenameout1}" ${species}_EM_ANTHRO
eod
#--run ferret script
ferret << eod 
go select_yr.jnl
exit
eod

#--replace undef with 0
cdo setmisstoc,0.0 ${filenameout1} ${filenameout1b}

#--remap to LMDz grid
#--OC to POM conversion factor
#--otherwise change to capital letters if not (eg NOx)
if [ ${species} == "OC"  ] ; then 
echo cdo remapcon,${gridfile} -chname,`echo ${species}_EM_ANTHRO | awk '{print toupper($0)}'`,flux -mulc,1.4 ${filenameout1b} ${filenameout2}
cdo remapcon,${gridfile} -chname,`echo ${species}_EM_ANTHRO | awk '{print toupper($0)}'`,flux -mulc,1.4 ${filenameout1b} ${filenameout2}
else
echo cdo remapcon,${gridfile} -chname,`echo ${species}_EM_ANTHRO | awk '{print toupper($0)}'`,flux  ${filenameout1b} ${filenameout2}
cdo remapcon,${gridfile} -chname,`echo ${species}_EM_ANTHRO | awk '{print toupper($0)}'`,flux ${filenameout1b} ${filenameout2}
fi

#--Improved Sheng & Zwiers algorithm + transform into vector
rm -f regrid.pro
cat << eod >> regrid.pro
pro regrid
filename='${filenameout2}'
print, filename
NETCDFREAD,filename,'flux',flux,dimflux
NETCDFREAD,filename,'lat',lat,dimlat0
NETCDFREAD,filename,'lon',lon,dimlon0
NETCDFREAD,filename,'TIME',time,dimtime0
dimlat=dimlat0(0)
dimlon=dimlon0(0)
dimtime=dimtime0(0)
print, 'dim flux=', dimflux
A = float([	[3./4., 1./8., 0., 0., 0., 0., 0., 0., 0., 0., 0., 1./8.],$
		[1./8., 3./4., 1./8., 0., 0., 0., 0., 0., 0., 0., 0., 0.],$
		[0., 1./8., 3./4., 1./8., 0., 0., 0., 0., 0., 0., 0., 0.],$
		[0., 0., 1./8., 3./4., 1./8., 0., 0., 0., 0., 0., 0., 0.],$
		[0., 0., 0., 1./8., 3./4., 1./8., 0., 0., 0., 0., 0., 0.],$
		[0., 0., 0., 0., 1./8., 3./4., 1./8., 0., 0., 0., 0., 0.],$
		[0., 0., 0., 0., 0., 1./8., 3./4., 1./8., 0., 0., 0., 0.],$
		[0., 0., 0., 0., 0., 0., 1./8., 3./4., 1./8., 0., 0., 0.],$
		[0., 0., 0., 0., 0., 0., 0., 1./8., 3./4., 1./8., 0., 0.],$
		[0., 0., 0., 0., 0., 0., 0., 0., 1./8., 3./4., 1./8., 0.],$
		[0., 0., 0., 0., 0., 0., 0., 0., 0., 1./8., 3./4., 1./8.],$
		[1./8., 0., 0., 0., 0., 0., 0., 0., 0., 0., 1./8., 3./4.]   ])
fluxinit=flux
flux_check=flux
for lo=0,dimlon-1 do begin
for la=0,dimlat-1 do begin
flux_check(lo,la,*) = invert(A) ## transpose(fluxinit(lo,la,*))
endfor
endfor			
m_bloq = make_array(dimlon,dimlat,12,value=0)		; Matrice booléenne "mois à bloquer ou non"
if total(where(flux_check lt 0)) ne -1 then m_bloq(where(flux_check lt 0)) = 1
; Correction/adaptation de la matrice S&Z en fonction du masque booléen m_bloq
for lo=0,dimlon-1 do begin
for la=0,dimlat-1 do begin
whereneg = where(flux_check(lo,la,*) lt 0)		; (12 pts max) Identification de potentiels points à problèmes, corrigés négativement
nbannul=n_elements(whereneg)*(total(whereneg) ne -1)
flux_corr=flux_check(lo,la,*)				; Création d'un vecteur pour recevoir les valeurs corrigées, initialisé à flux_check au cas où on n'ait rien à faire d'autre qu'une seule itération
A2 = A							; Je repars de la matrice A initiale, ce pour chaque point de grille
;							; Potentiellement plusieurs passages pour éliminer toutes les valeurs négatives
while nbannul ne 0 do begin				; Si l'on a effectivement des émissions corrigées négativement...
m_bloq(lo,la,whereneg) = 1				; Update de la matrice m_bloq
for m=0,11 do begin
if m eq 11 then begin					; Pour plus de facilité, mois précédents et suivants codés ici
p=10
s=0
endif else if m eq 0 then begin
p=11
s=1
endif else begin
p = m-1
s = m+1
endelse
if m_bloq(lo,la,m) then begin				; Je traite les mois bloqués en eux-mêmes
A2(p,m) = 0.
A2(m,m) = 1.
A2(s,m) = 0.
endif							; Fin du cas si l'on est sur un mois bloqué
if ~m_bloq(lo,la,m) then begin				; Je traite les mois non bloqués, pour ceux adjacents à un mois bloqué
if m_bloq(lo,la,p) and m_bloq(lo,la,s) then begin   		; Mois encadré de deux mois bloqués
A2(p,m) = 1./4.
A2(m,m) = 1./2.
A2(s,m) = 1./4.
endif else if m_bloq(lo,la,p) and ~m_bloq(lo,la,s) then begin	; Mois précédent bloqué (uniquement)
A2(p,m) = 2./8.
A2(m,m) = 5./8.
A2(s,m) = 1./8.
endif else if ~m_bloq(lo,la,p) and m_bloq(lo,la,s) then begin	; Mois suivant bloqué (uniquement)
A2(p,m) = 1./8.
A2(m,m) = 5./8.
A2(s,m) = 2./8.
endif
endif								; Fin du cas mois non bloqué
endfor								; Fin de la boucle sur les mois, balayage de la matrice
flux_corr = invert(A2) ## transpose(fluxinit(lo,la,*))		; Ré-itération de la multiplication matricielle avec la matrice A modifiée (A2)
whereneg = where(flux_corr lt 0)				; (12 pts max) Ré-identification de potentiels nouveaux points à problèmes, corrigés négativement
nbannul=n_elements(whereneg)*(total(whereneg) ne -1)
endwhile							; Fin du cas où l'on avait des problèmes d'émissions corrigées négativement
; *** IMPORTANT ! *** Pour signaler les mois bloqués, on prend la convention suivante :
;	valeur négative ou nulle <=> mois bloqué
;	valeur positive <=> mois à interpolation classique
flux(lo,la,*) = flux_corr					; En sortie de la boucle while, normalement flux_corr est complètement positif
endfor								; Fin boucle lat
endfor								; Fin boucle lon
nbnegtotal = n_elements(where(m_bloq eq 1)) * (total(where(m_bloq eq 1)) ne -1)
if nbnegtotal ne 0 then flux(where(m_bloq eq 1)) = -flux(where(m_bloq eq 1))		; Je force à des valeurs négatives
;
month_in_year=12
nbpoint=${nbpoint}
flux2=fltarr(nbpoint,month_in_year)
flux2(*,*)=0.0
;
for l=0,month_in_year-1 do begin
flux2(0,l)=TOTAL(flux(*,0,l))/float(dimlon)
flux2(nbpoint-1,l)=TOTAL(flux(*,dimlat-1,l))/float(dimlon)
endfor
;
pt=1
for j=1,dimlat-2  do begin
for i=0,dimlon-1  do begin
for l=0,month_in_year-1 do begin
  flux2(pt,l)=flux(i,j,l)
endfor
pt=pt+1
endfor
endfor
;
;saving netcdf file
;
fluxstruct={vector:fltarr(nbpoint),time:fltarr(month_in_year), $
            flx_${speciesinca}:fltarr(nbpoint,month_in_year) }
;
fluxstruct.vector=float(indgen(nbpoint)+1)
;;fluxstruct.time=float(indgen(month_in_year)+1)
fluxstruct.time=[15, 45, 75, 105, 135, 165, 195, 225, 255, 285, 315, 345]
fluxstruct.flx_${speciesinca}=flux2
;
attributes = {units:strarr(3),long_name:strarr(3)}
attributes.units = ['vector','days since 1960-01-01','flux']
attributes.long_name = ['vector', 'time', 'flux']
;
dimensions = {isdim:intarr(3), links:intarr(2,3)}
       dimensions.isdim =  [1,1,0]  ; (1=dimension, 0=variable)
       dimensions.links = [ [-1,-1],[-1,-1],[0,1]    ]
;
netcdfwrite,'${filenameout3}',fluxstruct,clobber=1, attributes=attributes, dimensions=dimensions
;
end
eod
#
#--calling IDL
#
/opt/idl-6.4/idl/bin/idl << eod
.r netcdf
.r netcdfwrite
.r struct_dims
.r regrid
regrid
exit
eod
#

#--cleaning the mess
rm -f ferret*
rm -f regrid.pro


#--now dealing with BB

#--input file
fileintbb=${dirout}${species}-em-openburning_input4MIPs_emissions_ScenarioMIP_IAMC-${prefix}-${scen}-1-1_gn_${year1}01-${year2}12_interp.nc
#--output files
filenameout1=${dirout}flux_${speciesinca}bb_${year}.nc
filenameout1b=${dirout}flux_0_${speciesinca}bb_${year}.nc
filenameout2=${dirout}flux_lmdz0_${speciesinca}bb_${year}.nc
filenameout3=${dirout}flux_vector_${speciesinca}bb_${year}.nc

echo ${fileintbb} ${filenameout1} ${filenameout1b} ${filenameout2} ${filenameout3}
rm -f ${filenameout1} ${filenameout1b} ${filenameout2} ${filenameout3}

#--unfortunately idl not happy with input netcdf files so need to ferretize files
#--I extract the correct year as well; no need to sum up sectors, as none in BB files
rm -f rewrite.jnl
cat << eod > rewrite.jnl
set memory/size=100
use "${fileintbb}"
set region/t=16-jan-${year}:16-dec-${year}
save/clobber/file="${filenameout1}" ${species}_EM_OPENBURNING
eod
#--run ferret script
ferret << eod
go rewrite.jnl
exit
eod

#--replace undef with 0 
cdo setmisstoc,0.0 ${filenameout1} ${filenameout1b}

#--remap to LMDz grid
#--OC to POM conversion factor
#--as ferret returns NOX, treat NOx NOX inconsistency in names by converting to upper case
if [ ${species} != "OC" ] ; then
echo cdo remapcon,${gridfile} -chname,`echo ${species}_EM_OPENBURNING | awk '{print toupper($0)}'`,flux ${filenameout1b} ${filenameout2}
cdo remapcon,${gridfile} -chname,`echo ${species}_EM_OPENBURNING | awk '{print toupper($0)}'`,flux ${filenameout1b} ${filenameout2}
else
echo cdo remapcon,${gridfile} -chname,${species}_EM_OPENBURNING,flux -mulc,1.4 ${filenameout1b} ${filenameout2}
cdo remapcon,${gridfile} -chname,${species}_EM_OPENBURNING,flux -mulc,1.4 ${filenameout1b} ${filenameout2}
fi

#--Improved Sheng & Zwiers algorithm + transform into vector
rm -f regrid.pro
cat << eod >> regrid.pro
pro regrid
filename='${filenameout2}'
print, filename
NETCDFREAD,filename,'flux',flux,dimflux
NETCDFREAD,filename,'lat',lat,dimlat0
NETCDFREAD,filename,'lon',lon,dimlon0
NETCDFREAD,filename,'TIME',time,dimtime0
dimlat=dimlat0(0)
dimlon=dimlon0(0)
dimtime=dimtime0(0)
print, 'dim flux=', dimflux
A = float([	[3./4., 1./8., 0., 0., 0., 0., 0., 0., 0., 0., 0., 1./8.],$
		[1./8., 3./4., 1./8., 0., 0., 0., 0., 0., 0., 0., 0., 0.],$
		[0., 1./8., 3./4., 1./8., 0., 0., 0., 0., 0., 0., 0., 0.],$
		[0., 0., 1./8., 3./4., 1./8., 0., 0., 0., 0., 0., 0., 0.],$
		[0., 0., 0., 1./8., 3./4., 1./8., 0., 0., 0., 0., 0., 0.],$
		[0., 0., 0., 0., 1./8., 3./4., 1./8., 0., 0., 0., 0., 0.],$
		[0., 0., 0., 0., 0., 1./8., 3./4., 1./8., 0., 0., 0., 0.],$
		[0., 0., 0., 0., 0., 0., 1./8., 3./4., 1./8., 0., 0., 0.],$
		[0., 0., 0., 0., 0., 0., 0., 1./8., 3./4., 1./8., 0., 0.],$
		[0., 0., 0., 0., 0., 0., 0., 0., 1./8., 3./4., 1./8., 0.],$
		[0., 0., 0., 0., 0., 0., 0., 0., 0., 1./8., 3./4., 1./8.],$
		[1./8., 0., 0., 0., 0., 0., 0., 0., 0., 0., 1./8., 3./4.]   ])
fluxinit=flux
flux_check=flux
for lo=0,dimlon-1 do begin
for la=0,dimlat-1 do begin
flux_check(lo,la,*) = invert(A) ## transpose(fluxinit(lo,la,*))
endfor
endfor			
m_bloq = make_array(dimlon,dimlat,12,value=0)		; Matrice booléenne "mois à bloquer ou non"
if total(where(flux_check lt 0)) ne -1 then m_bloq(where(flux_check lt 0)) = 1
; Correction/adaptation de la matrice S&Z en fonction du masque booléen m_bloq
for lo=0,dimlon-1 do begin
for la=0,dimlat-1 do begin
whereneg = where(flux_check(lo,la,*) lt 0)		; (12 pts max) Identification de potentiels points à problèmes, corrigés négativement
nbannul=n_elements(whereneg)*(total(whereneg) ne -1)
flux_corr=flux_check(lo,la,*)				; Création d'un vecteur pour recevoir les valeurs corrigées, initialisé à flux_check au cas où on n'ait rien à faire (à part 1 seule correction matricielle)
A2 = A							; Je repars de la matrice A initiale, ce pour chaque point de grille
;							; Potentiellement plusieurs passages pour éliminer toutes les valeurs négatives
while nbannul ne 0 do begin				; Si l'on a effectivement des émissions corrigées négativement...
m_bloq(lo,la,whereneg) = 1				; Update de la matrice m_bloq
for m=0,11 do begin
if m eq 11 then begin					; Pour plus de facilité, mois précédents et suivants codés ici
p=10
s=0
endif else if m eq 0 then begin
p=11
s=1
endif else begin
p = m-1
s = m+1
endelse
if m_bloq(lo,la,m) then begin				; Je traite les mois bloqués en eux-mêmes
A2(p,m) = 0.
A2(m,m) = 1.
A2(s,m) = 0.
endif							; Fin du cas si l'on est sur un mois bloqué
if ~m_bloq(lo,la,m) then begin				; Je traite les mois non bloqués, pour ceux adjacents à un mois bloqué
if m_bloq(lo,la,p) and m_bloq(lo,la,s) then begin   		; Mois encadré de deux mois bloqués
A2(p,m) = 1./4.
A2(m,m) = 1./2.
A2(s,m) = 1./4.
endif else if m_bloq(lo,la,p) and ~m_bloq(lo,la,s) then begin	; Mois précédent bloqué (uniquement)
A2(p,m) = 2./8.
A2(m,m) = 5./8.
A2(s,m) = 1./8.
endif else if ~m_bloq(lo,la,p) and m_bloq(lo,la,s) then begin	; Mois suivant bloqué (uniquement)
A2(p,m) = 1./8.
A2(m,m) = 5./8.
A2(s,m) = 2./8.
endif
endif								; Fin du cas mois non bloqué
endfor								; Fin de la boucle sur les mois, balayage de la matrice
flux_corr = invert(A2) ## transpose(fluxinit(lo,la,*))		; Ré-itération de la multiplication matricielle avec la matrice A modifiée (A2)
whereneg = where(flux_corr lt 0)				; (12 pts max) Ré-identification de potentiels nouveaux points à problèmes, corrigés négativement
nbannul=n_elements(whereneg)*(total(whereneg) ne -1)
endwhile							; Fin du cas où l'on avait des problèmes d'émissions corrigées négativement
; *** IMPORTANT ! *** Pour signaler les mois bloqués, on prend la convention suivante :
;	valeur négative ou nulle <=> mois bloqué
;	valeur positive <=> mois à interpolation classique
flux(lo,la,*) = flux_corr					; En sortie de la boucle while, normalement flux_corr est complètement positif
endfor								; Fin boucle lat
endfor								; Fin boucle lon
nbnegtotal = n_elements(where(m_bloq eq 1)) * (total(where(m_bloq eq 1)) ne -1)
if nbnegtotal ne 0 then flux(where(m_bloq eq 1)) = -flux(where(m_bloq eq 1))		; Je force à des valeurs négatives
;
month_in_year=12
nbpoint=${nbpoint}
flux2=fltarr(nbpoint,month_in_year)
flux2(*,*)=0.0
;
for l=0,month_in_year-1 do begin
flux2(0,l)=TOTAL(flux(*,0,l))/float(dimlon)
flux2(nbpoint-1,l)=TOTAL(flux(*,dimlat-1,l))/float(dimlon)
endfor
;
pt=1
for j=1,dimlat-2  do begin
for i=0,dimlon-1  do begin
for l=0,month_in_year-1 do begin
  flux2(pt,l)=flux(i,j,l)
endfor
pt=pt+1
endfor
endfor
;
;saving netcdf file
;
fluxstruct={vector:fltarr(nbpoint),time:fltarr(month_in_year), $
            flx_bb${speciesinca}:fltarr(nbpoint,month_in_year) }
;
fluxstruct.vector=float(indgen(nbpoint)+1)
;;fluxstruct.time=float(indgen(month_in_year)+1)
fluxstruct.time=[15, 45, 75, 105, 135, 165, 195, 225, 255, 285, 315, 345]
fluxstruct.flx_bb${speciesinca}=flux2
;
attributes = {units:strarr(3),long_name:strarr(3)}
attributes.units = ['vector','days since 1960-01-01','flux']
attributes.long_name = ['vector', 'time', 'flux']
;
dimensions = {isdim:intarr(3), links:intarr(2,3)}
       dimensions.isdim =  [1,1,0]  ; (1=dimension, 0=variable)
       dimensions.links = [ [-1,-1],[-1,-1],[0,1]    ]
;
netcdfwrite,'${filenameout3}',fluxstruct,clobber=1, attributes=attributes, dimensions=dimensions
;
end
eod
#
#--calling IDL
#
/opt/idl-6.4/idl/bin/idl << eod
.r netcdf
.r netcdfwrite
.r struct_dims
.r regrid
regrid
exit
eod
#

#--end loop on species
done

#--unfortunately idl use capital letters for variable names so need to change to small letters for now 
rm -f ${dirout}flux_vector_h2s_${year}.nc ${dirout}flux_vector_so4_${year}.nc
cdo expr,'flx_h2s=0.*FLX_SO2' ${dirout}flux_vector_so2_${year}.nc ${dirout}flux_vector_h2s_${year}.nc
cdo expr,'flx_so4=0.*FLX_SO2' ${dirout}flux_vector_so2_${year}.nc ${dirout}flux_vector_so4_${year}.nc

rm -f ${dirout}flux_vector_${year}.nc

#--combining everything into a single file with some final preprocessing
rm -f ${dirout}flux_vector_noxtot_${year}.nc ${dirout}flux_vector_so2tot_${year}.nc ${dirout}flux_vector_nh3tot_${year}.nc

#--deleting output file if already there
rm -f ${dirout}sflx_lmdz_cmip6_${year}.nc
#--merging all files into a single one
#--anthro NOx is NO2 so 30/46 scaling factor to change to NO 
#--openburning NOx is NO so no change in unit
cdo merge -expr,'flx_bc=FLX_BC' ${dirout}flux_vector_bc_${year}.nc -expr,'flx_bbbc=FLX_BBBC' ${dirout}flux_vector_bcbb_${year}.nc -expr,'flx_pom=FLX_POM' ${dirout}flux_vector_pom_${year}.nc -expr,'flx_bbpom=FLX_BBPOM' ${dirout}flux_vector_pombb_${year}.nc -expr,'flx_nox=30.*FLX_NOX/46.' ${dirout}flux_vector_nox_${year}.nc -expr,'flx_bbnox=FLX_BBNOX;' ${dirout}flux_vector_noxbb_${year}.nc -expr,'flx_so2=FLX_SO2' ${dirout}flux_vector_so2_${year}.nc -expr,'flx_bbso2=FLX_BBSO2' ${dirout}flux_vector_so2bb_${year}.nc -expr,'flx_nh3=FLX_NH3' ${dirout}flux_vector_nh3_${year}.nc -expr,'flx_bbnh3=FLX_BBNH3' ${dirout}flux_vector_nh3bb_${year}.nc  ${dirout}flux_vector_h2s_${year}.nc ${dirout}flux_vector_so4_${year}.nc -selname,conc_dms ${fileINCAex} ${dirout}sflx_lmdz_cmip6_${year}.nc

ncrename -d VECTOR,vector -v VECTOR,vector ${dirout}sflx_lmdz_cmip6_${year}.nc
ncrename -d TIME,time     -v TIME,time     ${dirout}sflx_lmdz_cmip6_${year}.nc

#--cleaning up 
#rm -f ${dirout}flux*_${year}.nc
#--to be uncommented in final version

#--end loop on years
done

rm -f ${fileintanthro} ${fileintbb}

#--end loop on scenarios
done

#--cleaning the mess
rm -f ferret*
rm -f regrid.pro
rm -f time_interp_over_gaps.pro sumsectors.jnl select_yr.jnl rewrite.jnl
